<?
/*
*  $Id: kick.inc,v 1.6 2004/12/28 23:24:05 bboyd Exp $
*  Written by Brian Boyd <bboyd@hotpop.com>
*/

# Defines all global functions.

define("KICK_INI", "/var/www/html/kickstart/main/kick.ini");
define("GLOBAL_INI", "/var/www/html/kickstart/main/global.ini");


// Get the value of a variable defined in the Kickstart
// config file.
function get_kick_var($section, $name) {
    if (! file_exists(KICK_INI) ) {
        crit_err("File " . KICK_INI . " not found");
    }

    $kick_conf = parse_ini_file(KICK_INI, true);

    if ($section) {
        if ( isset($kick_conf[$section][$name]) ) {
            return $kick_conf[$section][$name];
        }
    }
    else {
        foreach ($kick_conf as $sect) {
            foreach ($sect as $var => $val) {
                if ($var == $name) {
                    return $val;
                }
            }
        }
    }
    return 0;      // variable not found 
}

// Get the value of a variable defined in a host config file
function get_host_var($host, $section, $name) {
    $dir = get_kick_var("path", "base_path") . "/" .
           get_kick_var("path", "host_dir");

    if (! file_exists("$dir/$host.ini") ) {
        crit_err("File $dir/$host.ini not found");
    }

    $host_conf = parse_ini_file("$dir/$host.ini", true);

    // If a section was specified, go directly to it
    if ($section) {
        if ( isset($host_conf[$section][$name]) ) {
            return $host_conf[$section][$name];
        }
    }
    // Section not specified, must search for it
    else {
        foreach ($host_conf as $sect) {
            foreach ($sect as $var => $val) {
                if ($var == $name) {
                    return $val;
                }
            }
        }
    }

    // If the option $name was not found in the host config
    // try the group config
    if ($host_conf[group][group]) {
        return get_group_var($host_conf[group][group], $section, $name, $host);
    }
    return 0;    // variable not found
}

// Get the value of a variable defined in a group config file
function get_group_var($group, $section, $name, $host) {
    $dir = get_kick_var("path", "base_path") . "/" .
           get_kick_var("", "group_dir");

    if (! file_exists("$dir/$group.ini") ) {
        crit_err("File $dir/$group.ini not found");
    }

    $group_conf = parse_ini_file("$dir/$group.ini", true);

    if ($section) {
        if ( isset($group_conf[$section][$name]) ) {
            return $group_conf[$section][$name];
        }
    }
    else {
        foreach ($group_conf as $sect) {
            foreach ($sect as $var => $val) {
                if ($var == $name) {
                    return $val;
                }
            }
        }
    }

    // If the option $name was not found in the group config
    // try the os config
    return get_os_var(get_host_var($host, "software", "os"),
                      $section,
                      $name);
}

// Get the value of a variable defined in an OS config file
function get_os_var($os, $section, $name) {
    $dir = get_kick_var("path", "base_path") . "/" .
           get_kick_var("path", "os_dir");

    if (! file_exists("$dir/$os.ini") ) {
        crit_err("File $dir/$os.ini not found");
    }

    $os_conf = parse_ini_file("$dir/$os.ini", true);

    if ($section) {
        if ( isset($os_conf[$section][$name]) ) {
            return $os_conf[$section][$name];
        }
    }
    else {
        foreach ($os_conf as $sect) {
            foreach ($sect as $var => $val) {
                if ($var == $name) {
                    return $val;
                }
            }
        }
    }

    return 0;    // variable not found
}

// Get all of the variables and values in a section.
// If $exact_match is 0 (false), then all sections
// matching the pattern $section will be returned.
function get_section($host, $section, $exact_match) {

    // Read the host config file
    $dir = get_kick_var("path", "base_path") . "/" .
           get_kick_var("path", "host_dir");
    if (! file_exists("$dir/$host.ini") ) {
        crit_err("File $dir/$host.ini not found");
    }
    $host_conf = parse_ini_file("$dir/$host.ini", true);

    // Read the group config file
    $group = get_host_var($host, "group", "group");
    $dir = get_kick_var("path", "base_path") . "/" .
           get_kick_var("path", "group_dir");
    if (! file_exists("$dir/$group.ini") ) {
        crit_err("File $dir/$group.ini not found");
    }
    $group_conf = parse_ini_file("$dir/$group.ini", true);

    // Read the os config file
    $os = get_host_var($host, "software", "os");
    $dir = get_kick_var("path", "base_path") . "/" .
           get_kick_var("path", "os_dir");
    if (! file_exists("$dir/$os.ini") ) {
        crit_err("File $dir/$os.ini not found");
    }
    $os_conf = parse_ini_file("$dir/$os.ini", true);

    // Merge all config files together, with the precedence
    // in the following order: host, group, os
    $sect_matched = array();
    foreach ( array($os_conf, $group_conf, $host_conf) as $conf ) {
        if ($exact_match && $section && is_array($conf[$section])) {
            foreach ($conf[$section] as $name => $val) {
                $sect_matched[$section][$name] = $val;
            }
        }
        else {
            foreach ($conf as $sect_name => $sect_val) {
                if ( preg_match("/$section/", $sect_name) ) {
                    if (! is_array($conf[$sect_name]) ) {
                        $sect_matched[$sect_name] = array();
                    }

                    foreach ($sect_val as $name => $val) {
                        if (! is_array($conf[$sect_name][$name]) ) {
                            $sect_matched[$sect_name][$name] = array();
                        }
                        $sect_matched[$sect_name][$name] = $val;
                    }
                }
            }
        }
    }

    return $sect_matched;
}

// Get the value of a variable defined in the global
// config file.
function get_global_var($section, $name) {
    if (! file_exists(GLOBAL_INI) ) {
        crit_err("File " . GLOBAL_INI . " not found");
    }

    $kick_conf = parse_ini_file(GLOBAL_INI, true);

    if ($section) {
        if ( isset($kick_conf[$section][$name]) ) {
            return $kick_conf[$section][$name];
        }
    }
    else {
        foreach ($kick_conf as $sect) {
            foreach ($sect as $var => $val) {
                if ($var == $name) {
                    return $val;
                }
            }
        }
    }
    return 0;
}

function crit_err($err_msg) {
    print "$err_msg\n";
    exit;
}

function warn_err($err_msg) {
    print "$err_msg\n";
    exit;
}

?>
